var servicesModule = angular.module('myApp.services');

servicesModule.service('paymentOnGo', function(
    $filter,
    $interval,
    $ionicLoading,
    $ionicModal,
    $q,
    alertPopup,
    appConfig,
    datetimeFormatter,
    protocol,
    radiotaxiUpdating,
    trips,
    webCommunication
) {
    var modalPage = null;
    var currentCodeExpiration = 0;
    var expirationInterval = null;

    var getTitle = function() {
        var title = appConfig.IMAGE_BASE_URL + 'logo-apptaxi-pay.svg';
        return title;
    };

    var formatExpiration = function(expiration) {
        var date = new Date(expiration * 1000);
        var result = datetimeFormatter.formatTime(date);
        return result;
    };

    var getLogoImg = function() {
        return radiotaxiUpdating.getRadioTaxiImg();
    };

    var calcRemainingCodeExpirationSeconds = function(exp) {
        return exp - Date.now() / 1000;
    };

    var resetModalCode = function() {
        if (modalPage) {
            modalPage.code = null;
            return true; // code reset
        }
        return false; // nothing reset
    };

    var isCodeExpired = function() {
        return currentCodeExpiration <= 0;
    };

    var setExpirationTimer = function(exp) {
        if (expirationInterval !== null) {
            $interval.cancel(expirationInterval);
        }
        expirationInterval = $interval(function() {
            currentCodeExpiration = calcRemainingCodeExpirationSeconds(exp);

            if (isCodeExpired()) {
                $interval.cancel(expirationInterval);
            }
        }, 5000);
    };

    var hideAndResetCode = function() {
        this.hide() // modalPage
            .then(resetModalCode);
    };

    var loadCode = function() {
        /* Setup */

        var MAX_RETRIES_ON_FAILURE = 3;
        var currentRetries = 0;

        /* Actions */

        resetModalCode();
        performPaymentQuickRequest(manageQuickRequestMessage);

        /* Functions */

        function performPaymentQuickRequest(cb) {
            protocol.createPaymentQuickRequestMessage().then(cb);
        }

        function manageQuickRequestMessage(message) {
            var onSendSuccess = function(result) {
                Debug.Log('MYAJAX <<: ' + JSON.stringify(result));

                $ionicLoading.hide();

                if (
                    protocol.isResultError(result) ||
                    !protocol.isResultPositive(result, message)
                ) {
                    alertPopup.show({
                        title: $filter('translate')('Error'),
                        message: result.errore || result.messaggio,
                        callback: hideAndResetCode.bind(modalPage),
                    });

                    return;
                }

                var resultExpiration = result.parametri.scadenza;

                modalPage.code = result.parametri.codice;
                modalPage.expiration = formatExpiration(resultExpiration);

                currentCodeExpiration = calcRemainingCodeExpirationSeconds(
                    resultExpiration
                );
                setExpirationTimer(resultExpiration);
            };

            var onSendFailure = function(jqXHR, textStatus, errorThrown) {
                defaultErrorManager();
            };

            var onSendException = function(err) {
                Debug.Log('trips.update - send exception' + err.toString());
                defaultErrorManager();
            };

            Debug.Log('MY AJAX >>: ' + JSON.stringify(message));

            webCommunication.send(
                message,
                onSendSuccess,
                onSendFailure,
                onSendException
            );
        }

        function defaultErrorManager() {
            currentRetries += 1;

            if (currentRetries > MAX_RETRIES_ON_FAILURE) {
                alertPopup.show({
                    title: $filter('translate')('Error'),
                    message: $filter('translate')('serverCommunicationFailed'),
                    callback: hideAndResetCode.bind(modalPage),
                });

                return;
            }

            performPaymentQuickRequest(manageQuickRequestMessage);
        }
    };

    var checkIfUserIsAllowedToRequestACode = function() {
        var warnInCaseOfError = false;
        var doNotShowLoadingIcon = true;
        var forceUpdate = true;

        var deferred = $q.defer();

        trips.update(
            function() {
                if (
                    trips.getOrdersToPayNumber() >
                    appConfig.MAX_NUMBER_OF_ORDERS_TO_PAY_ALLOWED_WHEN_ASKING_FOR_A_NEW_CAB
                ) {
                    alertPopup.show({
                        title: $filter('translate')('Error'),
                        message: $filter('translate')('thereAreOrdersToPay'),
                        callback: hideAndResetCode.bind(modalPage),
                    });

                    deferred.reject();
                    return;
                }

                deferred.resolve();
            },
            warnInCaseOfError,
            doNotShowLoadingIcon,
            forceUpdate
        );

        return deferred.promise;
    };

    this.create = function(scope) {
        if (modalPage) {
            return;
        }

        $ionicModal
            .fromTemplateUrl('templates/payment-on-go/payment-on-go.html', {
                scope: scope,
            })
            .then(function(modal) {
                modalPage = modal;
                modalPage.getTitle = getTitle;
                modalPage.loadCode = loadCode;
                modalPage.getLogoImg = getLogoImg;
                modalPage.isCodeExpired = isCodeExpired;
                modalPage.hideAndResetCode = hideAndResetCode;

                // POG = payment on go
                scope.modalPOG = modalPage;
            });

        scope.$on('$destroy', function() {
            modalPage.remove();
            modalPage = null;
        });
    };

    this.show = function() {
        modalPage
            .show()
            .then(resetModalCode)
            .then(checkIfUserIsAllowedToRequestACode)
            .then(loadCode);
    };
});
