(function() {
    'use strict';

    angular
        .module('myApp.services')
        .service('priceQuotationModal', priceQuotationModalService);

    function priceQuotationModalService(
        $ionicModal,
        $q,
        priceQuotation,
        radiotaxiUpdating
    ) {
        /* Setup */

        var modalPage = null;
        var modalDeferred;

        var quotationInfo = {};
        var quotation = [];

        var data = {
            totalQuotationPrices: [],
            distance: 0,
            options: {
                passengers: 1,
                baggages: 0,
                type: 'diurnal',
            },
            isHoliday: priceQuotation.isTodayHoliday(),
            nocturnalHours: {
                startHour: 22,
                endHour: 6,
            },
        };

        /* Service */

        angular.extend(this, {
            create: create,
            show: show,
        });

        /* Service methods */

        function create(scope) {
            if (modalPage) {
                return $q.resolve();
            }

            scope.$on('$destroy', function() {
                modalPage.remove();
                modalPage = null;
            });

            return $ionicModal
                .fromTemplateUrl(
                    'templates/price-quotation/price-quotation.html',
                    {
                        scope: scope,
                    }
                )
                .then(function(modal) {
                    modalPage = modal;

                    modalPage.getTotalQuotationPrices = getTotalQuotationPrices;
                    modalPage.calculateOptionsCost = calculateOptionsCost;
                    modalPage.getBaggageCost = getBaggageCost;
                    modalPage.getCallCost = getCallCost;
                    modalPage.getMinPassengers = getMinPassengers;
                    modalPage.getPassengersCost = getPassengersCost;
                    modalPage.cancel = cancel;
                    modalPage.proceed = proceed;

                    modalPage.data = data;

                    scope.modalPC = modalPage;
                });
        }

        function show() {
            quotationInfo = radiotaxiUpdating.getPriceQuotationInfo();
            quotation = priceQuotation.getQuotation();

            data.distance = quotation.distance;
            data.options.type = quotation.activeType;

            data.isHoliday = priceQuotation.isTodayHoliday();

            data.nocturnalHours = calculateNocturnalTime();

            modalPage.show();

            modalDeferred = $q.defer();

            return modalDeferred.promise;
        }

        /* Modal methods */

        function cancel() {
            hideAndResolve(false);
        }

        function proceed() {
            updateLatestTotalQuotationPrices();
            hideAndResolve(true);
        }

        function getTotalQuotationPrices() {
            // questa brutta cosa serve per evitare gli errori di "Infinite $digest Loop"
            // che avverrebbero tornando ogni volta un array nuovo od oggetti diversi allo
            // stesso indice. Per utilizzare direttamente data.totalQuotationPrices nell'ng-repeat
            // bisognerebbe cambiare la logica e chiamare _getTotalQuotationPrices in ogni eventuale
            // punto dove andrebbero ricalcolati i prezzi (e.g. ad ogni change di passengers/baggages)
            var totalQuotationPrices = priceQuotation.getQuotationWithOptionsCostCalculated(
                data.options
            )[data.options.type];

            totalQuotationPrices.forEach(function(quote, index) {
                if (data.totalQuotationPrices[index]) {
                    angular.extend(data.totalQuotationPrices[index], quote);
                } else {
                    data.totalQuotationPrices.push(quote);
                }
            });

            return data.totalQuotationPrices;
        }

        function calculateOptionsCost() {
            var activeType = data.options.type;
            var supplements = quotationInfo[activeType].supplements;

            return priceQuotation.calculateOptionsCost(
                supplements,
                data.options
            );
        }

        function getBaggageCost() {
            return quotationInfo[data.options.type].supplements.baggage;
        }

        function getCallCost() {
            return quotationInfo[data.options.type].supplements.call;
        }

        function getMinPassengers() {
            return quotationInfo[data.options.type].supplements.minPassengers;
        }

        function getPassengersCost() {
            return quotationInfo[data.options.type].supplements.person;
        }

        // Il notturno è fisso, per semplificare calcoliamo una volta all'entrata e basta
        function calculateNocturnalTime() {
            var diurnal = quotationInfo.diurnal;
            var startTime = diurnal.timestart;
            var endTime = diurnal.timeend;

            var roundHour = function(time) {
                var momentTime = moment(time);
                var roundedHourTime =
                    momentTime.minutes() >= 30
                        ? momentTime.add(1, 'hour').startOf('hour')
                        : momentTime.startOf('hour');
                return roundedHourTime.hour();
            };

            return {
                startHour: roundHour(endTime),
                endHour: roundHour(startTime),
            };
        }

        /* Utils */

        function hideAndResolve(response) {
            modalPage.hide().then(function() {
                modalDeferred.resolve(response);
            });
        }

        function updateLatestTotalQuotationPrices() {
            var prices = getTotalQuotationPrices().map(function (quote) {
                return quote.price;
            });
            priceQuotation.setLatestTotalQuotationPrices(prices);
        }
    }
})();
